@@ -1,4 +1,4 @@
-/* $Id: netload.c,v 1.30 2004/07/22 13:52:59 bdejean Exp $ */
+/* $Id: netload.c,v 1.33 2004/11/28 01:32:55 bdejean Exp $ */
 
 /* Copyright (C) 1998-99 Martin Baulig
    This file is part of LibGTop 1.0.
@@ -25,6 +25,11 @@
 #include <glibtop/error.h>
 #include <glibtop/netload.h>
 
+#include "glibtop_private.h"
+
+#include <errno.h>
+#include <string.h>
+
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/ioctl.h>
@@ -47,7 +52,9 @@
 #include <linux/udp.h>
 #endif
 
-/* IPV6 */
+
+#ifdef HAVE_IFADDRS_H
+/* needed for IPV6 support */
 
 #include <ifaddrs.h>
 
@@ -56,7 +63,8 @@
    (((((__const uint8_t *) (a))[0] & 0xff) == 0x3f   \
      || (((__const uint8_t *) (a))[0] & 0xff) == 0x20))
 #endif
-/* IPV6 */
+#endif /* HAVE_IFADDRS_H */
+
 
 #define _GLIBTOP_IP_FW_ACCTIN	0x1000	/* Account incoming packets only. */
 #define _GLIBTOP_IP_FW_ACCTOUT	0x2000	/* Account outgoing packets only. */
@@ -113,6 +121,69 @@ glibtop_init_netload_s (glibtop *server)
 	_glibtop_sysdeps_netload_packets;
 }
 
+
+#ifdef HAVE_IFADDRS_H
+
+static void get_ipv6(glibtop *server, glibtop_netload *buf,
+		     const char *interface)
+{
+	struct ifaddrs *ifa0, *ifr6;
+
+	if(getifaddrs (&ifa0) != 0)
+	{
+		glibtop_warn_r(server, "getifaddrs failed : %s", strerror(errno));
+		return;
+	}
+
+	for (ifr6 = ifa0; ifr6; ifr6 = ifr6->ifa_next) {
+		if (strcmp (ifr6->ifa_name, interface) == 0
+		    && ifr6->ifa_addr != NULL
+		    && ifr6->ifa_addr->sa_family == AF_INET6)
+			break;
+	}
+
+	if(!ifr6) goto free_ipv6;
+
+	memcpy(buf->address6,
+	       &((struct sockaddr_in6 *) ifr6->ifa_addr)->sin6_addr,
+	       16);
+
+	memcpy(buf->prefix6,
+	       &((struct sockaddr_in6 *) ifr6->ifa_netmask)->sin6_addr,
+	       16);
+
+
+	if (IN6_IS_ADDR_LINKLOCAL (buf->address6))
+		buf->scope6 = GLIBTOP_IF_IN6_SCOPE_LINK;
+
+	else if (IN6_IS_ADDR_SITELOCAL (buf->address6))
+		buf->scope6 = GLIBTOP_IF_IN6_SCOPE_SITE;
+
+	else if (IN6_IS_ADDR_GLOBAL (buf->address6)
+		 || IN6_IS_ADDR_MC_ORGLOCAL (buf->address6)
+		 || IN6_IS_ADDR_V4COMPAT (buf->address6)
+		 || IN6_IS_ADDR_MULTICAST (buf->address6)
+		 || IN6_IS_ADDR_UNSPECIFIED (buf->address6)
+		)
+		buf->scope6 = GLIBTOP_IF_IN6_SCOPE_GLOBAL;
+
+	else if (IN6_IS_ADDR_LOOPBACK (buf->address6))
+		buf->scope6 = GLIBTOP_IF_IN6_SCOPE_HOST;
+
+	else
+		buf->scope6 = GLIBTOP_IF_IN6_SCOPE_UNKNOWN;
+
+	buf->flags |= _glibtop_sysdeps_netload_6;
+
+ free_ipv6:
+	freeifaddrs(ifa0);
+}
+
+#endif /* HAVE_IFADDRS_H */
+
+
+
+
 /* Provides network statistics. */
 
 void
@@ -373,52 +444,10 @@ glibtop_get_netload_s (glibtop *server, glibtop_netload *buf,
 
     fclose (f);
 
-
-    /* IPv6 */
-    {
-	    struct ifaddrs *ifa0, *ifr6;
-	    getifaddrs (&ifa0);
-
-	    for (ifr6 = ifa0; ifr6; ifr6 = ifr6->ifa_next) {
-		    if (strcmp (ifr6->ifa_name, interface) == 0
-			&& ifr6->ifa_addr->sa_family == AF_INET6)
-			    break;
-	    }
-
-	    if(!ifr6) goto free_ipv6;
-
-	    memcpy(buf->address6,
-		   &((struct sockaddr_in6 *) ifr6->ifa_addr)->sin6_addr,
-		   16);
-
-	    memcpy(buf->prefix6,
-		   &((struct sockaddr_in6 *) ifr6->ifa_netmask)->sin6_addr,
-		   16);
-
-
-	    if (IN6_IS_ADDR_LINKLOCAL (buf->address6))
-		    buf->scope6 = GLIBTOP_IF_IN6_SCOPE_LINK;
-
-	    else if (IN6_IS_ADDR_SITELOCAL (buf->address6))
-		    buf->scope6 = GLIBTOP_IF_IN6_SCOPE_SITE;
-
-	    else if (IN6_IS_ADDR_GLOBAL (buf->address6)
-		     || IN6_IS_ADDR_MC_ORGLOCAL (buf->address6)
-		     || IN6_IS_ADDR_V4COMPAT (buf->address6)
-		     || IN6_IS_ADDR_MULTICAST (buf->address6)
-		     || IN6_IS_ADDR_UNSPECIFIED (buf->address6)
-		    )
-		    buf->scope6 = GLIBTOP_IF_IN6_SCOPE_GLOBAL;
-
-	    else if (IN6_IS_ADDR_LOOPBACK (buf->address6))
-		    buf->scope6 = GLIBTOP_IF_IN6_SCOPE_HOST;
-
-	    else
-		    buf->scope6 = GLIBTOP_IF_IN6_SCOPE_UNKNOWN;
-
-	    buf->flags |= _glibtop_sysdeps_netload_6;
-
-    free_ipv6:
-	    freeifaddrs(ifa0);
-    } /* IPV6 */
+#ifdef HAVE_IFADDRS_H
+    get_ipv6(server, buf, interface);
+#endif /* HAVE_IFADDRS_H */
 }
+
+
+
